import torch
import numpy as np

from delitoolkit.encoding_utils import SbertPredictor


class ProbingTargetPredictor(torch.nn.Module):
    def __init__(self, num_of_classes):
        super(ProbingTargetPredictor, self).__init__()
        self.pre_classifier = torch.nn.Linear(768, 512)
        self.dropout = torch.nn.Dropout(0.2)

        self.classifier = torch.nn.Linear(512, num_of_classes)

    def forward(self, vector_representation):
        pooler = self.pre_classifier(vector_representation)
        pooler = self.dropout(pooler)
        output = self.classifier(pooler)
        return output


class DeliAnnotationPredictor:
    def __init__(self, type_path='models/predicting_type.zip',
                 probing_target_path='models/predicting_probing_target_delidata20rc.zip',
                 nonprobing_target_path='models/predicting_nonprobing_target_delidata20rc.zip') -> None:
        self.embedder = SbertPredictor('gtr-t5-xl')

        device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")

        self.type_model = ProbingTargetPredictor(num_of_classes=3)
        self.probing_target_model = ProbingTargetPredictor(num_of_classes=3)
        self.nonprobing_target_model = ProbingTargetPredictor(num_of_classes=4)

        self.type_model.load_state_dict(torch.load(type_path, map_location=device))
        self.probing_target_model.load_state_dict(torch.load(probing_target_path, map_location=device))
        self.nonprobing_target_model.load_state_dict(torch.load(nonprobing_target_path, map_location=device))
        self.type_model.eval()
        self.probing_target_model.eval()
        self.nonprobing_target_model.eval()

        self.type_reverse_dict = {0: 'Probing', 1: 'Non-probing-deliberation', 2: '0'}
        self.probing_target_reverse_dict = {0: 'Moderation', 1: 'Reasoning', 2: 'Solution'}
        self.non_probing_reverse_dict = {0: 'Reasoning', 1: 'Solution', 2: 'Agree', 3: 'Disagree'}

    def predict(self, item):
        torch.random.manual_seed(42)
        encoded = torch.tensor(np.array(self.embedder.encode(item)))

        with torch.no_grad():
            type_prediction = self.type_reverse_dict[np.argmax(self.type_model.forward(encoded)).item()]
            if type_prediction == "0":
                return type_prediction, ''
            elif type_prediction == "Probing":
                target_predictio = self.probing_target_reverse_dict[
                    np.argmax(self.probing_target_model.forward(encoded)).item()]
            elif type_prediction == "Non-probing-deliberation":
                target_predictio = self.non_probing_reverse_dict[
                    np.argmax(self.nonprobing_target_model.forward(encoded)).item()]

            return type_prediction, target_predictio



import torch
from sentence_transformers import SentenceTransformer

class SbertPredictor():
    def __init__(self, model='all-distilroberta-v1'):
        self.model = SentenceTransformer(model)
        self.model.eval()
    def tokenize_and_predict(self, text):
        with torch.no_grad():
            return self.model.encode(text)

    def encode(self, text):
        with torch.no_grad():
            return self.model.encode(text)



import pickle


class BoWSimple:
    def __init__(self):
        with open('../models/bow_full_delidata.model', 'rb') as f:
            self.model = pickle.load(f)

    def predict_proba(self, dialogue_context):
        extended_context = ['BEGIN', *dialogue_context]
        bigrams = [p + "<SEP>" + n for p, n in zip(extended_context[:-1], extended_context[1:])]
        predictions = self.model.predict_proba(bigrams)

        return list(zip(dialogue_context, [p[1] for p in predictions]))



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as reqs:
    requirements = reqs.read()

test_requirements = [ ]

setup(
    author="Georgi Karadzhov",
    author_email='georgi.m.karadjov@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A toolkit for evaluating deliberative discussions and building DEliBots. For more details see delibot.xyz",
    install_requires=["sentence-transformers>=2.2.2","torch>=2.1.2","transformers>=4.20.1","numpy>=1.24.4","scikit-learn==1.2.2"],
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='delitoolkit',
    name='delitoolkit',
    packages=find_packages(include=['delitoolkit', 'delitoolkit.*']),
    package_data={'delitoolkit': ['models', 'delidata']},
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/gkaradzhov/delitoolkit',
    version='0.3.3',
    zip_safe=False,
)



import csv
import os


class DeliData:
    def __init__(self):
        files = os.listdir('delidata_corpus')
        self.corpus = {}
        for f in files:
            if f == '__init__.py':
                continue
            with open('delidata_corpus/' + f, 'r', encoding="utf8") as rf:
                a = [{k: v for k, v in row.items()} for row in
                     csv.DictReader(rf, delimiter='\t', skipinitialspace=True)]
                self.corpus[f] = a



